@@ -90,7 +90,8 @@ module Y2Storage
         device.public_send(meth, partition_section.public_send(usage_attr)) if device.respond_to?(meth)
         return if ignored_attrs.empty?
 
-        issues_list.add(:conflicting_attrs, partition_section, usage_attr, ignored_attrs)
+        issues_list.add(Y2Storage::AutoinstIssues::ConflictingAttrs,
+          partition_section, usage_attr, ignored_attrs)
       end
 
       # @return [Array<Symbol>] List of 'usage' attributes. The list is ordered by precedence.
@@ -168,7 +169,7 @@ module Y2Storage
           end
 
         if error
-          issues_list.add(:invalid_encryption, partition_section, error)
+          issues_list.add(Y2Storage::AutoinstIssues::InvalidEncryption, partition_section, error)
           return
         end
 
@@ -182,7 +183,7 @@ module Y2Storage
       # @return [String,nil]
       def find_encryption_password(partition_section)
         if partition_section.crypt_key.nil? || partition_section.crypt_key.empty?
-          issues_list.add(:missing_value, partition_section, :crypt_key)
+          issues_list.add(Y2Storage::AutoinstIssues::MissingValue, partition_section, :crypt_key)
           return
         end
         partition_section.crypt_key
@@ -238,6 +239,32 @@ module Y2Storage
           else
             []
           end
+
+        configure_btrfs_quotas(device, section)
+      end
+
+      # Sets the Btrfs quotas according to the section and the subvolumes
+      #
+      # If `section.quotas` is nil, it inspect whether quotas are needed for any
+      # of the subvolumes. In that case, it sets `device.quota` to true.
+      #
+      # @param device  [Planned::Device] Planned device
+      # @param section [AutoinstProfile::PartitionSection] AutoYaST specification
+      def configure_btrfs_quotas(device, section)
+        if !section.quotas.nil?
+          device.quota = section.quotas
+          return
+        end
+
+        subvols_with_quotas = device.subvolumes.select do |subvol|
+          subvol.referenced_limit && !subvol.referenced_limit.unlimited?
+        end
+        return if subvols_with_quotas.empty?
+
+        device.quota = true
+        issues_list.add(
+          Y2Storage::AutoinstIssues::MissingBtrfsQuotas, section, subvols_with_quotas
+        )
       end
 
       # Return the default subvolume attributes for a given mount point
@@ -286,14 +313,24 @@ module Y2Storage
         !!spec && spec.btrfs_read_only?
       end
 
-      # @param planned_device [Planned::Partition,Planned::LvmLV,Planned::Md] Planned device
+      # @param planned_device [Planned::Device] Planned device
       # @param device         [Y2Storage::Device] Device to reuse
       # @param section        [AutoinstProfile::PartitionSection] AutoYaST specification
       def add_device_reuse(planned_device, device, section)
-        planned_device.uuid = section.uuid
         planned_device.reuse_name = device.is_a?(LvmVg) ? device.volume_group_name : device.name
-        planned_device.reformat = !!section.format
+        planned_device.uuid = section.uuid
         planned_device.resize = !!section.resize if planned_device.respond_to?(:resize=)
+        config_filesystem_reuse(planned_device, device, section)
+      end
+
+      # @param planned_device [Planned::Device] Planned device
+      # @param device         [Y2Storage::Device] Device to reuse
+      # @param section        [AutoinstProfile::PartitionSection,AutoinstProfile::Drive] relevant section
+      #   of the AutoYaST specification
+      def config_filesystem_reuse(planned_device, device, section)
+        return unless section.is_a?(AutoinstProfile::PartitionSection)
+
+        planned_device.reformat = !!section.format
         check_reusable_filesystem(planned_device, device, section) if device.respond_to?(:filesystem)
       end
 
@@ -306,7 +343,7 @@ module Y2Storage
         # really needed, e.g. reusing a bios_boot partition (bsc#1134330)
         return if planned_device.mount_point.nil? && planned_device.filesystem_type.nil?
 
-        issues_list.add(:missing_reusable_filesystem, section)
+        issues_list.add(Y2Storage::AutoinstIssues::MissingReusableFilesystem, section)
       end
 
       # Parse the 'size' element
@@ -357,11 +394,11 @@ module Y2Storage
           elsif part_section.label
             disk.partitions.find { |i| i.filesystem_label == part_section.label }
           else
-            issues_list.add(:missing_reuse_info, part_section)
+            issues_list.add(Y2Storage::AutoinstIssues::MissingReuseInfo, part_section)
             nil
           end
 
-        issues_list.add(:missing_reusable_device, part_section) unless device
+        issues_list.add(Y2Storage::AutoinstIssues::MissingReusableDevice, part_section) unless device
         device
       end
 
@@ -393,7 +430,7 @@ module Y2Storage
         size_info = parse_size(part_section, PARTITION_MIN_SIZE, DiskSize.unlimited)
 
         if size_info.nil?
-          issues_list.add(:invalid_value, part_section, :size)
+          issues_list.add(Y2Storage::AutoinstIssues::InvalidValue, part_section, :size)
           return false
         end
 
